@@ -249,9 +249,7 @@ class MultiMaterial(Material):
         hash : int
           Xor hash of the contained materials.
         """
-        hashed = int(np.bitwise_xor.reduce([hash(m) for m in self.materials]))
-
-        return hashed
+        return int(np.bitwise_xor.reduce([hash(m) for m in self.materials]))
 
     def __iter__(self):
         return iter(self.materials)
@@ -1007,7 +1005,7 @@ def pack(
                     scale = max_tex_size.max() / max_tex_size_individual
                     max_tex_size = np.round(max_tex_size / scale).astype(np.int64)
 
-                unpadded_sizes.append(max_tex_size)
+                unpadded_sizes.append(tuple(max_tex_size))
 
             # use the same size for all of them to ensure
             # that texture atlassing is identical
